/**
far away - a free space shooter game
Copyright (C) 2005  Alexander Clausen, Dirk Rathlev, Stefan Toller

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include <SDL.h>
//#include "SDL_mixer.h"
#include "faraway.h"

extern faMain* g_pfaMain;

//Mix_Chunk *sounds_loaded[NUM_SOUNDS];

int Sound::sound_init()
{
	int i;
	
	for (i=0; i<NUM_SOUNDS; i++)
	{
		sounds_loaded[i] = NULL;
	}
	
	music = NULL;
	
	// open audio device 
	int audio_rate = g_pfaMain->Configs.nSampleRate;
	Uint16 audio_format = AUDIO_S16; /* 16-bit stereo */
	int audio_channels = 2;
	int audio_buffers = 2048;

  	/* This is where we open up our audio device.  Mix_OpenAudio takes
	as its parameters the audio format we'd /like/ to have. */
	if(Mix_OpenAudio(audio_rate, audio_format, audio_channels, audio_buffers)) {
		printf("Unable to open audio!\n");
		exit(1);
	}

	
	// load a test sounds

	load_sound(1,"sound/laser2.wav");
	
	//play_music("music/music1.ogg");

	return 0;
}

int Sound::exit_sound()
{
	// stop music
	Mix_HaltMusic();
		
	if (g_pfaMain->Configs.bMusicEnabled) {
		// clean up
		Mix_CloseAudio();
	}
	return 0;
}

int Sound::load_sound(int nSound, const char *file)
{
	sounds_loaded[nSound] = Mix_LoadWAV(file);

	printf("soundfile %s loaded\n",file);
	
	return 0;
}
	
int Sound::play_sound(int nSound)
{
	if (g_pfaMain->Configs.bSoundEnabled) {
		Mix_PlayChannel(-1, sounds_loaded[nSound], 0);
	}
	return 0;
}

void musicDone()
{
	Mix_HaltMusic();
	Mix_FreeMusic(g_pfaMain->g_pSound->music);
	g_pfaMain->g_pSound->music = NULL;
}

int Sound::play_music(const char *file)
{
	if ( (g_pfaMain->Configs.bSoundEnabled) && (g_pfaMain->Configs.bMusicEnabled) ) {
		// load music
		music = Mix_LoadMUS(file);
	
		// play music one time
		Mix_PlayMusic(music, 0);
	
		// callback to clean up after playing music
		Mix_HookMusicFinished(musicDone);
	}
	
	return 0;
}
