/**
far away - a free space shooter game
Copyright (C) 2005  Alexander Clausen, Dirk Rathlev, Stefan Toller

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include "SDL.h"
#include "SDL_image.h"

#include "faraway.h"

extern faMain* g_pfaMain;

int UI::UI_init()
{
	Uint32 colorkey;
	int i;
	
	splash_screens[0] = IMG_Load("graphics/ui_gameover.png");
	splash_screens[1] = IMG_Load("graphics/ui_leveldone.png");
	splash_screens[2] = IMG_Load("graphics/ui_campaigndone.png");
	
	// check whether all images were succesful loaded
	// if yes, transform them into a faster format
	for (i=0; i<MAX_SPLASH_SCREENS; i++)
	{
		if (splash_screens[i] == NULL)
		{
			fprintf (stderr, "splash screen %d not found\n",i, SDL_GetError ());
			return 1;
		}
		else
		{
			colorkey = SDL_MapRGB(splash_screens[i]->format, 199, 245, 6);
			SDL_SetColorKey(splash_screens[i],SDL_SRCCOLORKEY|SDL_RLEACCEL|SDL_HWACCEL,colorkey);
			splash_screens[i] = SDL_DisplayFormatAlpha(splash_screens[i]);
		}
	}
	printf("splash screens sucessful loaded\n");
	
	ui_graphics[0] = IMG_Load("graphics/menu_bg.jpg");
	ui_graphics[1] = IMG_Load("graphics/ui_hpbar.png");
	ui_graphics[2] = IMG_Load("graphics/menu_frame.png");
	ui_graphics[3] = IMG_Load("graphics/ui_splashbg.jpg");
	
	// check whether all images were succesful loaded
	// if yes, transform them into a faster format
	for (i=0; i<MAX_UI_GRAPHICS; i++)
	{
		if (ui_graphics[i] == NULL)
		{
			fprintf (stderr, "UI graphic %d not found\n",i, SDL_GetError ());
			return 1;
		}
		else
		{
			colorkey = SDL_MapRGB(ui_graphics[i]->format, 199, 245, 6);
			SDL_SetColorKey( ui_graphics[i],SDL_SRCCOLORKEY|SDL_RLEACCEL|SDL_HWACCEL, colorkey );
			ui_graphics[i] = SDL_DisplayFormatAlpha(ui_graphics[i]);
		}
	}
	printf("ui graphics sucessful loaded\n");
	
	menu_icons[0] = IMG_Load("graphics/menu_start.png");
	menu_icons[1] = IMG_Load("graphics/menu_ende.png");
	
	// check whether all images were succesful loaded
	for (i=0; i<MAX_MENU_ICONS; i++)
	{
		if (menu_icons[i] == NULL)
		{
			fprintf (stderr, "menu image %d not found\n",i, SDL_GetError ());
			return 1;
		}
		{
			colorkey = SDL_MapRGB(menu_icons[i]->format, 199, 245, 6);
			SDL_SetColorKey( menu_icons[i],SDL_SRCCOLORKEY|SDL_RLEACCEL|SDL_HWACCEL, colorkey );
			menu_icons[i] = SDL_DisplayFormatAlpha(menu_icons[i]);
		}
	}
	printf("icons sucessful loaded\n");
	
		// initialize start menu
	for (i=0; i<NUM_START_MENU_ITEMS; i++)
	{
		StartMenuItems[i].bvisible = false;
		StartMenuItems[i].x = 0;
		StartMenuItems[i].y = 0;
		StartMenuItems[i].w = 128;
		StartMenuItems[i].h = 64;
		StartMenuItems[i].nImage = 0;
	}
	
	// set some buttons
	StartMenuItems[0].bvisible = true;
	StartMenuItems[0].x = (SCREEN_W/2)-(StartMenuItems[0].w/2);
	StartMenuItems[0].y = 150;
	StartMenuItems[0].nImage = 0;
	StartMenuItems[0].nType = MENU_START;
	
	StartMenuItems[1].bvisible = true;
	StartMenuItems[1].x = (SCREEN_W/2)-(StartMenuItems[1].w/2);
	StartMenuItems[1].y = 250;
	StartMenuItems[1].nImage = 1;
	StartMenuItems[1].nType = MENU_QUIT;
	
	// initialize ingame menu
	for (i=0; i<NUM_INGAME_MENU_ITEMS; i++)
	{
		IngameMenuItems[i].bvisible = false;
		IngameMenuItems[i].x = 0;
		IngameMenuItems[i].y = 0;
		IngameMenuItems[i].w = 128;
		IngameMenuItems[i].h = 64;
		IngameMenuItems[i].nImage = 0;
	}
	
	// set some buttons
	IngameMenuItems[0].bvisible = true;
	IngameMenuItems[0].x = (SCREEN_W/2)-(IngameMenuItems[0].w/2);
	IngameMenuItems[0].y = 150;
	IngameMenuItems[0].nImage = 0;
	IngameMenuItems[0].nType = MENU_RETURN;
	
	IngameMenuItems[1].bvisible = true;
	IngameMenuItems[1].x = (SCREEN_W/2)-(IngameMenuItems[0].w/2);
	IngameMenuItems[1].y = 250;
	IngameMenuItems[1].nImage = 1;
	IngameMenuItems[1].nType = MENU_QUIT;
	
	nStartact = 0;
	nIngameact = 0;
	
	return 0;
}

int UI::exit_UI()
{
	int i;
	// free surfaces
	for (i=0; i<MAX_MENU_ICONS; i++)
	{
		if (menu_icons[i])
		{
			SDL_FreeSurface(menu_icons[i]);
		}
	}
	
	for (i=0; i<MAX_UI_GRAPHICS; i++)
	{
		if (ui_graphics[i])
		{
			SDL_FreeSurface(ui_graphics[i]);
		}
	}
	
	for (i=0; i<MAX_SPLASH_SCREENS; i++)
	{
		if (splash_screens[i])
		{
			SDL_FreeSurface(splash_screens[i]);
		}
	}
	
	return 0;
}

int UI::process_start_menu()
{
	int i;
		
	if ( (g_pfaMain->g_pInput->keys[SDLK_DOWN]==SDL_PRESSED) && (g_pfaMain->g_pInput->ok) )
	{
		g_pfaMain->g_pInput->ok = false;
		nStartact++;
		if (nStartact >= NUM_START_MENU_ITEMS)
		{
			nStartact = 0;
		}
		printf("nStartact: %d\n",nStartact);
	}
	if ( (g_pfaMain->g_pInput->keys[SDLK_UP]==SDL_PRESSED) && (g_pfaMain->g_pInput->ok) )
	{
		g_pfaMain->g_pInput->ok = false;
		nStartact--;
		if (nStartact < 0)
		{
			nStartact = NUM_START_MENU_ITEMS-1;
		}
		printf("nStartact: %d\n",nStartact);
	}
	
	for (i=0; i<NUM_START_MENU_ITEMS; i++)
	{
		if (check_mouse_over(i))
		{
			nStartact = i;
		}
	}
	if ( (g_pfaMain->g_pInput->Mouse.buttonl) || ((g_pfaMain->g_pInput->keys[SDLK_RETURN]==SDL_PRESSED) && (g_pfaMain->g_pInput->ok)) )
	{
		g_pfaMain->g_pInput->ok = false;
		switch (StartMenuItems[nStartact].nType)
		{
			case MENU_START:
			{
				if (g_pfaMain->g_pCampaign->load_campaign("campaigns/campaign1.fac"))
				{
					printf("campaign load failed\n");
					return 1;
				}
				printf("campaign succesful loaded\n");
				printf("%s\n",g_pfaMain->g_pCampaign->level_paths[0]);
				g_pfaMain->way_passed = 0.0f;
				g_pfaMain->GameState = GS_GAME;
			}
			break;
			case MENU_QUIT:
			{
				g_pfaMain->GameState = GS_QUIT;
			}
			break;
		}
	}
	return 0;
}

int UI::process_ingame_menu()
{
	int i;
	
	if ( (g_pfaMain->g_pInput->keys[SDLK_DOWN]==SDL_PRESSED) && (g_pfaMain->g_pInput->ok) )
	{
		g_pfaMain->g_pInput->ok = false;
		nIngameact++;
		if (nIngameact >= NUM_INGAME_MENU_ITEMS)
		{
			nIngameact = 0;
		}
		printf("nIngameact: %d\n",nIngameact);
	}
	if ( (g_pfaMain->g_pInput->keys[SDLK_UP]==SDL_PRESSED) && (g_pfaMain->g_pInput->ok) )
	{
		g_pfaMain->g_pInput->ok = false;
		nIngameact--;
		if (nIngameact < 0)
		{
			nIngameact = NUM_INGAME_MENU_ITEMS-1;
		}
		printf("nIngameact: %d\n",nIngameact);
	}
		
	for (i=0; i<NUM_INGAME_MENU_ITEMS; i++)
	{
		if (check_mouse_over(i))
		{
			nIngameact = i;
		}
			
	}
	if ( (g_pfaMain->g_pInput->Mouse.buttonl) || ((g_pfaMain->g_pInput->keys[SDLK_RETURN]==SDL_PRESSED) && (g_pfaMain->g_pInput->ok)) )
	{
		g_pfaMain->g_pInput->ok = false;
		switch (IngameMenuItems[nIngameact].nType)
		{
			case MENU_RETURN:
			{
				g_pfaMain->GameState = GS_GAME;
			}
			break;
			case MENU_QUIT:
			{
				g_pfaMain->GameState = GS_QUIT;
			}
			break;
		}
	}
	return 0;
}

int UI::render_start_menu(SDL_Surface * screen)
{
	SDL_Rect dest,src;
	int i;
	
	// clear surface
	SDL_FillRect(screen,NULL,SDL_MapRGB(screen->format, 199, 245, 6));
	
	// render background
	src.x = 0;
	src.y = 0;
	src.w = 800;
	src.h = 600;
	
	dest.x = 0;
	dest.y = 0;
	dest.w = SCREEN_W;
	dest.h = SCREEN_H;
	
	if (SDL_BlitSurface(ui_graphics[0], &src, screen, &dest))
	{
		printf("error while drawing menu background\n");
	}
	
	// render decorations
	src.x = 0;
	src.y = 0;
	src.w = 196;
	src.h = 256;
	
	dest.x = StartMenuItems[0].x-((196-StartMenuItems[0].w)/2);
	dest.y = StartMenuItems[0].y-32;
	dest.w = 196;
	dest.h = 256;
	
	if (SDL_BlitSurface(ui_graphics[2], &src, screen, &dest))
	{
		printf("error while drawing menu frame\n");
	}
	
	for (i=0; i<NUM_START_MENU_ITEMS; i++)
	{
		//if (g_pObjects->lObjects[i].nTypeIndex != 0)
		if (StartMenuItems[i].bvisible)
		{
			if (i == nStartact)
			{
				src.x = 0;
				src.y = 0;
				src.w = StartMenuItems[i].w;
				src.h = StartMenuItems[i].h;
			}
			else
			{
				src.x = StartMenuItems[i].w;
				src.y = 0;
				src.w = StartMenuItems[i].w;
				src.h = StartMenuItems[i].h;
			}

			
			dest.x = StartMenuItems[i].x;
			dest.y = StartMenuItems[i].y;
			dest.w = StartMenuItems[i].w;
			dest.h = StartMenuItems[i].h;
			
			if (SDL_BlitSurface(menu_icons[StartMenuItems[i].nImage], &src, screen, &dest))
			{
				printf("error while drawing menu icons\n");
			}
		}
	}

	// change backbuffer and screen
	SDL_Flip (screen);
	return 0;
}

int UI::render_ingame_menu(SDL_Surface * screen)
{
	SDL_Rect dest,src;
	int i;
	
	// clear surface
	SDL_FillRect(screen,NULL,SDL_MapRGB(screen->format, 0, 0, 0));
	
	// render background
	src.x = 0;
	src.y = 0;
	src.w = 800;
	src.h = 600;
	
	dest.x = 0;
	dest.y = 0;
	dest.w = SCREEN_W;
	dest.h = SCREEN_H;
	
	if (SDL_BlitSurface(ui_graphics[0], &src, screen, &dest))
	{
		printf("error while drawing menu background\n");
	}
	
	// render decorations
	src.x = 0;
	src.y = 0;
	src.w = 196;
	src.h = 256;
	
	dest.x = IngameMenuItems[0].x-((196-IngameMenuItems[0].w)/2);
	dest.y = IngameMenuItems[0].y-32;
	dest.w = 196;
	dest.h = 256;
	
	if (SDL_BlitSurface(ui_graphics[2], &src, screen, &dest))
	{
		printf("error while drawing menu frame\n");
	}
	
	for (i=0; i<NUM_INGAME_MENU_ITEMS; i++)
	{
		if (IngameMenuItems[i].bvisible)
		{
			if (i == nIngameact)
			{
				src.x = 0;
				src.y = 0;
				src.w = IngameMenuItems[i].w;
				src.h = IngameMenuItems[i].h;
			}
			else
			{
				src.x = IngameMenuItems[i].w+1;
				src.y = 0;
				src.w = IngameMenuItems[i].w;
				src.h = IngameMenuItems[i].h;
			}
			
			dest.x = IngameMenuItems[i].x;
			dest.y = IngameMenuItems[i].y;
			dest.w = IngameMenuItems[i].w;
			dest.h = IngameMenuItems[i].h;
			
			if (SDL_BlitSurface(menu_icons[IngameMenuItems[i].nImage], &src, screen, &dest))
			{
				printf("error while drawing menu icons\n");
			}
		}
	}

	// change backbuffer and screen
	SDL_Flip (screen);
	return 0;
}

int UI::render_ui(SDL_Surface * screen)
{
	int maxhp,hp;
	float quot;
	SDL_Rect dest,src;
	
	quot = 0.0f;
	
	maxhp = g_pfaMain->g_pObjects->lObjects[g_pfaMain->g_pObjects->num_player1].ObjType->maxhp;
	hp = g_pfaMain->g_pObjects->lObjects[g_pfaMain->g_pObjects->num_player1].hp;
	if (hp > 0)
	{
		quot = ((float)hp/(float)maxhp);
	}
	// render hp bar
	src.x = 0;
	src.y = 0;
	src.w = (int)(quot*128.0f);
	src.h = 16;
	
	dest.x = 0;
	dest.y = 0;
	dest.w = (int)(quot*128.0f);
	dest.h = 16;
	
	if (SDL_BlitSurface(ui_graphics[1], &src, screen, &dest))
	{
		printf("error while drawing hp bar\n");
	}
	
	return 0;
}

int UI::check_mouse_over(int nItem)
{
	if (g_pfaMain->GameState == GS_START_MENU)
	{
		if ( (g_pfaMain->g_pInput->Mouse.x >= StartMenuItems[nItem].x) && (g_pfaMain->g_pInput->Mouse.x <= StartMenuItems[nItem].x+StartMenuItems[nItem].w) && (g_pfaMain->g_pInput->Mouse.y >= StartMenuItems[nItem].y) && (g_pfaMain->g_pInput->Mouse.y <= StartMenuItems[nItem].y+StartMenuItems[nItem].h) )
		{
			return 1;
		}
	}
	else if (g_pfaMain->GameState == GS_MENU)
	{
		if ( (g_pfaMain->g_pInput->Mouse.x >= IngameMenuItems[nItem].x) && (g_pfaMain->g_pInput->Mouse.x <= IngameMenuItems[nItem].x+IngameMenuItems[nItem].w) && (g_pfaMain->g_pInput->Mouse.y >= IngameMenuItems[nItem].y) && (g_pfaMain->g_pInput->Mouse.y <= IngameMenuItems[nItem].y+IngameMenuItems[nItem].h) )
		{
			return 1;
		}
	}
	return 0;
}

int UI::show_splashscreen(SDL_Surface * screen, int index)
{
	SDL_Rect dest,src;
	
	// clear surface
	SDL_FillRect(screen,NULL,SDL_MapRGB(screen->format, 0, 0, 0));
	
	// render background
	src.x = 0;
	src.y = 0;
	src.w = 800;
	src.h = 600;
	
	dest.x = 0;
	dest.y = 0;
	dest.w = SCREEN_W;
	dest.h = SCREEN_H;
	
	if (SDL_BlitSurface(ui_graphics[3], &src, screen, &dest))
	{
		printf("error while drawing splash background\n");
	}
	
	src.x = 0;
	src.y = 0;
	src.w = 680;
	src.h = 495;
	
	dest.x = (SCREEN_W/2)-340;
	dest.y = (SCREEN_H/2)-248;
	dest.w = 680;
	dest.h = 495;
	
	if (SDL_BlitSurface(splash_screens[index], &src, screen, &dest))
	{
		printf("error while drawing splash screen %d\n",index);
	}
	
	// change backbuffer and screen
	SDL_Flip (screen);
	
	return 0;
}





